@@ -1,22 +1,26 @@
 /* adapter_float.c - psycopg pfloat type wrapper implementation
  *
- * Copyright (C) 2003-2009 Federico Di Gregorio <fog@debian.org>
+ * Copyright (C) 2003-2010 Federico Di Gregorio <fog@debian.org>
  *
  * This file is part of psycopg.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2,
- * or (at your option) any later version.
+ * psycopg2 is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published
+ * by the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * In addition, as a special exception, the copyright holders give
+ * permission to link this program with the OpenSSL library (or with
+ * modified versions of OpenSSL that use the same license as OpenSSL),
+ * and distribute linked combinations including the two.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You must obey the GNU Lesser General Public License in all respects for
+ * all of the code used other than OpenSSL.
+ *
+ * psycopg2 is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ * License for more details.
  */
 
 #define PY_SSIZE_T_CLEAN
@@ -44,13 +48,12 @@ pfloat_str(pfloatObject *self)
     else if (isinf(n))
         return PyString_FromString("'Infinity'::float");
     else
-        return PyObject_Str(self->wrapped);
+        return PyObject_Repr(self->wrapped);
 }
 
 static PyObject *
 pfloat_getquoted(pfloatObject *self, PyObject *args)
 {
-    if (!PyArg_ParseTuple(args, "")) return NULL;
     return pfloat_str(self);
 }
 
@@ -82,7 +85,7 @@ static struct PyMemberDef pfloatObject_members[] = {
 /* object method table */
 
 static PyMethodDef pfloatObject_methods[] = {
-    {"getquoted", (PyCFunction)pfloat_getquoted, METH_VARARGS,
+    {"getquoted", (PyCFunction)pfloat_getquoted, METH_NOARGS,
      "getquoted() -> wrapped object value as SQL-quoted string"},
     {"__conform__", (PyCFunction)pfloat_conform, METH_VARARGS, NULL},
     {NULL}  /* Sentinel */
@@ -242,5 +245,5 @@ psyco_Float(PyObject *module, PyObject *args)
     if (!PyArg_ParseTuple(args, "O", &obj))
         return NULL;
 
-    return PyObject_CallFunction((PyObject *)&pfloatType, "O", obj);
+    return PyObject_CallFunctionObjArgs((PyObject *)&pfloatType, obj, NULL);
 }
