@@ -32,7 +32,10 @@
 #include "psycopg/psycopg.h"
 #include "psycopg/connection.h"
 #include "psycopg/cursor.h"
+#include "psycopg/green.h"
 #include "psycopg/lobject.h"
+#include "psycopg/notify.h"
+#include "psycopg/xid.h"
 #include "psycopg/typecast.h"
 #include "psycopg/microprotocols.h"
 #include "psycopg/microprotocols_proto.h"
@@ -45,21 +48,18 @@
 #include "psycopg/adapter_asis.h"
 #include "psycopg/adapter_list.h"
 #include "psycopg/typecast_binary.h"
+#include "psycopg/typecast_datetime.h"
 
 #ifdef HAVE_MXDATETIME
 #include <mxDateTime.h>
 #include "psycopg/adapter_mxdatetime.h"
-HIDDEN mxDateTimeModule_APIObject *mxDateTimeP = NULL;
+#include "psycopg/typecast_mxdatetime.h"
 #endif
 
 /* some module-level variables, like the datetime module */
 #include <datetime.h>
 #include "psycopg/adapter_datetime.h"
 HIDDEN PyObject *pyDateTimeModuleP = NULL;
-HIDDEN PyObject *pyDateTypeP = NULL;
-HIDDEN PyObject *pyTimeTypeP = NULL;
-HIDDEN PyObject *pyDateTimeTypeP = NULL;
-HIDDEN PyObject *pyDeltaTypeP = NULL;
 
 /* pointers to the psycopg.tz classes */
 HIDDEN PyObject *pyPsycopgTzModule = NULL;
@@ -90,6 +90,7 @@ HIDDEN int psycopg_debug_enabled = 0;
 "- ``user`` -- user name used to authenticate\n"                           \
 "- ``password`` -- password used to authenticate\n"                        \
 "- ``sslmode`` -- SSL mode (see PostgreSQL documentation)\n\n"             \
+"- ``async`` -- if the connection should provide asynchronous API\n\n"     \
 "If the ``connection_factory`` keyword argument is not provided this\n"    \
 "function always return an instance of the `connection` class.\n"          \
 "Else the given sub-class of `extensions.connection` will be used to\n"    \
@@ -118,14 +119,16 @@ psyco_connect(PyObject *self, PyObject *args, PyObject *keywds)
     const char *database=NULL, *user=NULL, *password=NULL;
     const char *host=NULL, *sslmode=NULL;
     char port[16];
+    int async = 0;
 
     static char *kwlist[] = {"dsn", "database", "host", "port",
                              "user", "password", "sslmode",
-                             "connection_factory", NULL};
+                             "connection_factory", "async", NULL};
 
-    if (!PyArg_ParseTupleAndKeywords(args, keywds, "|sssOsssO", kwlist,
+    if (!PyArg_ParseTupleAndKeywords(args, keywds, "|sssOsssOi", kwlist,
                                      &dsn_static, &database, &host, &pyport,
-                                     &user, &password, &sslmode, &factory)) {
+                                     &user, &password, &sslmode,
+                                     &factory, &async)) {
         return NULL;
     }
 
@@ -190,11 +193,22 @@ psyco_connect(PyObject *self, PyObject *args, PyObject *keywds)
 
     {
       const char *dsn = (dsn_static != NULL ? dsn_static : dsn_dynamic);
-      Dprintf("psyco_connect: dsn = '%s'", dsn);
+      Dprintf("psyco_connect: dsn = '%s', async = %d", dsn, async);
 
       /* allocate connection, fill with errors and return it */
       if (factory == NULL) factory = (PyObject *)&connectionType;
-      conn = PyObject_CallFunction(factory, "s", dsn);
+      /* Here we are breaking the connection.__init__ interface defined
+       * by psycopg2. So, if not requiring an async conn, avoid passing
+       * the async parameter. */
+      /* TODO: would it be possible to avoid an additional parameter
+       * to the conn constructor? A subclass? (but it would require mixins
+       * to further subclass) Another dsn parameter (but is not really
+       * a connection parameter that can be configured) */
+      if (!async) {
+        conn = PyObject_CallFunction(factory, "s", dsn);
+      } else {
+        conn = PyObject_CallFunction(factory, "si", dsn, async);
+      }
     }
 
     goto cleanup;
@@ -295,20 +309,20 @@ psyco_adapters_init(PyObject *mod)
     /* the module has already been initialized, so we can obtain the callable
        objects directly from its dictionary :) */
     call = PyMapping_GetItemString(mod, "DateFromPy");
-    microprotocols_add((PyTypeObject*)pyDateTypeP, NULL, call);
+    microprotocols_add(PyDateTimeAPI->DateType, NULL, call);
     call = PyMapping_GetItemString(mod, "TimeFromPy");
-    microprotocols_add((PyTypeObject*)pyTimeTypeP, NULL, call);
+    microprotocols_add(PyDateTimeAPI->TimeType, NULL, call);
     call = PyMapping_GetItemString(mod, "TimestampFromPy");
-    microprotocols_add((PyTypeObject*)pyDateTimeTypeP, NULL, call);
+    microprotocols_add(PyDateTimeAPI->DateTimeType, NULL, call);
     call = PyMapping_GetItemString(mod, "IntervalFromPy");
-    microprotocols_add((PyTypeObject*)pyDeltaTypeP, NULL, call);
+    microprotocols_add(PyDateTimeAPI->DeltaType, NULL, call);
 
 #ifdef HAVE_MXDATETIME
     /* as above, we use the callable objects from the psycopg module */
     call = PyMapping_GetItemString(mod, "TimestampFromMx");
-    microprotocols_add(mxDateTimeP->DateTime_Type, NULL, call);
+    microprotocols_add(mxDateTime.DateTime_Type, NULL, call);
     call = PyMapping_GetItemString(mod, "TimeFromMx");
-    microprotocols_add(mxDateTimeP->DateTimeDelta_Type, NULL, call);
+    microprotocols_add(mxDateTime.DateTimeDelta_Type, NULL, call);
 #endif
 }
 
@@ -679,6 +693,13 @@ static PyMethodDef psycopgMethods[] = {
      METH_VARARGS, psyco_IntervalFromMx_doc},
 #endif
 
+#ifdef PSYCOPG_EXTENSIONS
+    {"set_wait_callback",  (PyCFunction)psyco_set_wait_callback,
+     METH_O, psyco_set_wait_callback_doc},
+    {"get_wait_callback",  (PyCFunction)psyco_get_wait_callback,
+     METH_NOARGS, psyco_get_wait_callback_doc},
+#endif
+
     {NULL, NULL, 0, NULL}        /* Sentinel */
 };
 
@@ -710,6 +731,8 @@ init_psycopg(void)
     asisType.ob_type       = &PyType_Type;
     listType.ob_type       = &PyType_Type;
     chunkType.ob_type      = &PyType_Type;
+    NotifyType.ob_type     = &PyType_Type;
+    XidType.ob_type        = &PyType_Type;
 
     if (PyType_Ready(&connectionType) == -1) return;
     if (PyType_Ready(&cursorType) == -1) return;
@@ -723,6 +746,8 @@ init_psycopg(void)
     if (PyType_Ready(&asisType) == -1) return;
     if (PyType_Ready(&listType) == -1) return;
     if (PyType_Ready(&chunkType) == -1) return;
+    if (PyType_Ready(&NotifyType) == -1) return;
+    if (PyType_Ready(&XidType) == -1) return;
 
 #ifdef PSYCOPG_EXTENSIONS
     lobjectType.ob_type    = &PyType_Type;
@@ -738,7 +763,8 @@ init_psycopg(void)
         PyErr_SetString(PyExc_ImportError, "can't import mx.DateTime module");
         return;
     }
-    mxDateTimeP = &mxDateTime;
+    if (psyco_adapter_mxdatetime_init()) { return; }
+    if (psyco_typecast_mxdatetime_init()) { return; }
 #endif
 
     /* import python builtin datetime module, if available */
@@ -748,16 +774,15 @@ init_psycopg(void)
         PyErr_SetString(PyExc_ImportError, "can't import datetime module");
         return;
     }
+
+    /* Initialize the PyDateTimeAPI everywhere is used */
+    PyDateTime_IMPORT;
+    if (psyco_adapter_datetime_init()) { return; }
+    if (psyco_typecast_datetime_init()) { return; }
+
     pydatetimeType.ob_type = &PyType_Type;
     if (PyType_Ready(&pydatetimeType) == -1) return;
 
-    /* now we define the datetime types, this is crazy because python should
-       be doing that, not us! */
-    pyDateTypeP = PyObject_GetAttrString(pyDateTimeModuleP, "date");
-    pyTimeTypeP = PyObject_GetAttrString(pyDateTimeModuleP, "time");
-    pyDateTimeTypeP = PyObject_GetAttrString(pyDateTimeModuleP, "datetime");
-    pyDeltaTypeP = PyObject_GetAttrString(pyDateTimeModuleP, "timedelta");
-
     /* import psycopg2.tz anyway (TODO: replace with C-level module?) */
     pyPsycopgTzModule = PyImport_ImportModule("psycopg2.tz");
     if (pyPsycopgTzModule == NULL) {
@@ -797,6 +822,8 @@ init_psycopg(void)
     PyModule_AddObject(module, "connection", (PyObject*)&connectionType);
     PyModule_AddObject(module, "cursor", (PyObject*)&cursorType);
     PyModule_AddObject(module, "ISQLQuote", (PyObject*)&isqlquoteType);
+    PyModule_AddObject(module, "Notify", (PyObject*)&NotifyType);
+    PyModule_AddObject(module, "Xid", (PyObject*)&XidType);
 #ifdef PSYCOPG_EXTENSIONS
     PyModule_AddObject(module, "lobject", (PyObject*)&lobjectType);
 #endif
@@ -828,6 +855,8 @@ init_psycopg(void)
     listType.tp_alloc = PyType_GenericAlloc;
     chunkType.tp_alloc = PyType_GenericAlloc;
     pydatetimeType.tp_alloc = PyType_GenericAlloc;
+    NotifyType.tp_alloc = PyType_GenericAlloc;
+    XidType.tp_alloc = PyType_GenericAlloc;
 
 #ifdef PSYCOPG_EXTENSIONS
     lobjectType.tp_alloc = PyType_GenericAlloc;
